/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CreateDiskImageDialog.h"
#include "QDebug.h"
#include "QConfig.h"
#include "DialogHelper.h"
#include "VolumeConfig.h"

#include "Pline/FSysUtil.h"

#include <glibmm/i18n.h>
#include <assert.h>

//==============================================================================

CreateDiskImageDialog::CreateDiskImageDialog(
		BaseObjectType* cobject,
		const Glib::RefPtr<Gtk::Builder>& refGlade)
: QDialog(cobject)
{
	set_modal(true);
	set_title(_("Create Encrypted Disk Image"));
	
	gpoDialogHelper = 0;
	gpoVolumeConfig = 0;
	gpvecVolumeConfigLabel = 0;
	
	UtilGui::mapCreateDiskImageDlg(refGlade, gtCreateDiskImageDlg);	
	
	// tCreatePlainDiskImageDlgItem
	gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poCreateButton->signal_clicked().connect(
		sigc::mem_fun(*this, &CreateDiskImageDialog::onCreate));

	gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poCancelButton->signal_clicked().connect(
		sigc::mem_fun(*this, &CreateDiskImageDialog::onCancel));
		
	gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poSaveAsBrowseButton->signal_clicked().connect(
		sigc::mem_fun(*this, &CreateDiskImageDialog::onSaveAs));
		
	Gtk::ComboBoxText* pocmbFSys = manage(new Gtk::ComboBoxText);
			
	gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poFSysVBox->pack_start(*pocmbFSys, false, false);
	gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poFSysComboBox = pocmbFSys;
	
	Gtk::ComboBoxText* pocmbPlaceAfter = manage(new Gtk::ComboBoxText);
			
	gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poPlaceAfterVBox->pack_start(*pocmbPlaceAfter, false, false);
	gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poPlaceAfterComboBox = pocmbPlaceAfter;
		
	// local
	gtCreateDiskImageDlg.poUsePasswordRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &CreateDiskImageDialog::onUsePassword));
	
	gtCreateDiskImageDlg.poUseKeyFileRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &CreateDiskImageDialog::onUseKeyFile));
		
	Gtk::ComboBoxText* pocmbCipher = manage(new Gtk::ComboBoxText);
			
	gtCreateDiskImageDlg.poCipherVBox->pack_start(*pocmbCipher, false, false);
	gtCreateDiskImageDlg.poCipherComboBox = pocmbCipher;
	
	Gtk::ComboBoxText* pocmbCBC = manage(new Gtk::ComboBoxText);
			
	gtCreateDiskImageDlg.poCBCVBox->pack_start(*pocmbCBC, false, false);
	gtCreateDiskImageDlg.poCBCComboBox = pocmbCBC;
	
	Gtk::ComboBoxText* pocmbKeySize = manage(new Gtk::ComboBoxText);
			
	gtCreateDiskImageDlg.poKeySizeVBox->pack_start(*pocmbKeySize, false, false);
	gtCreateDiskImageDlg.poKeySizeComboBox = pocmbKeySize;
	
	show_all_children();
}

//==============================================================================

CreateDiskImageDialog::~CreateDiskImageDialog()
{
	if (gpoDialogHelper)
		delete gpoDialogHelper;
}

//==============================================================================

void CreateDiskImageDialog::buildFSys()
{
	unsigned int xui;

	gvecFSys.clear();
	
	gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poFSysComboBox->clear_items();
	
	Pline::FSysUtil::getMkFSysList(&gvecFSys);

	for (xui=0; xui<gvecFSys.size(); ++xui)
		gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poFSysComboBox->append_text(gvecFSys[xui]);
}

//==============================================================================

void CreateDiskImageDialog::onCancel()
{
	QDialog::response(Gtk::RESPONSE_CANCEL);
}

//==============================================================================

void CreateDiskImageDialog::onCreate()
{
	updateData(false);
	
	if (!validate())
		return;
		
	QDialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void CreateDiskImageDialog::onSaveAs()
{
	gpoDialogHelper->onSaveAs();
}

//==============================================================================

void CreateDiskImageDialog::onUseKeyFile()
{
	gpoDialogHelper->onUseKeyFile();
}

//==============================================================================

void CreateDiskImageDialog::onUsePassword()
{
	gpoDialogHelper->onUsePassword();
}

//==============================================================================

void CreateDiskImageDialog::setData(VolumeConfig *poVolumeConfig,
									std::vector<Glib::ustring> *pvecVolumeConfigLabel,
									unsigned long long *pullVolumeSize,
									bool *pbIsFullAllocation,
									Glib::ustring *pstCipher,
									Glib::ustring *pstCBC,
									Glib::ustring *pstKeySize)
{
	gpoVolumeConfig = poVolumeConfig;
	gpvecVolumeConfigLabel = pvecVolumeConfigLabel;
	
	buildFSys();
	
	//
	// Dialog Helper
	//
	
	if (gpoDialogHelper)
		delete gpoDialogHelper;

	gpoDialogHelper = new DialogHelper(this, gpoVolumeConfig);
	
	// mount
	gpoDialogHelper->setMkFSysComboBox(gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poFSysComboBox);
	gpoDialogHelper->setFSysList(&gvecFSys);
	gpoDialogHelper->setPlaceAfterComboBox(gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poPlaceAfterComboBox);
	gpoDialogHelper->setPlaceAfterList(gpvecVolumeConfigLabel);
	gpoDialogHelper->setSaveAsEntry(gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poSaveAsEntry);
	gpoDialogHelper->setVolumeLabelEntry(gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poVolumeLabelEntry);
	gpoDialogHelper->setKeyFileFileChooser(gtCreateDiskImageDlg.poKeyFileFileChooser);
	gpoDialogHelper->setUseKeyFileRadioButton(gtCreateDiskImageDlg.poUseKeyFileRadioButton);
	gpoDialogHelper->setUsePasswordRadioButton(gtCreateDiskImageDlg.poUsePasswordRadioButton);
	gpoDialogHelper->setEnableEncryption(true);
	
	// new disk image
	gpoDialogHelper->setVolumeSizeEntry(gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poVolumeSizeEntry,
										pullVolumeSize);
	
	gpoDialogHelper->setSizeRadioButton(gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poMRadioButton,
										gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poGRadioButton);
	
	gpoDialogHelper->setAllocRadioButton(	gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poAllocFullRadioButton,
											gtCreateDiskImageDlg.tCreatePlainDiskImageDlgItem.poAllocCompactRadioButton,
											pbIsFullAllocation);
											
	gpoDialogHelper->setCipherComboBox(	gtCreateDiskImageDlg.poCipherComboBox,
										pstCipher);
	gpoDialogHelper->setCBCComboBox(gtCreateDiskImageDlg.poCBCComboBox,
									pstCBC);
	gpoDialogHelper->setKeySizeComboBox(gtCreateDiskImageDlg.poKeySizeComboBox,
										pstKeySize);
	
	gpoDialogHelper->buildPlaceAfter();
	gpoDialogHelper->buildCiphers();
		
	updateData(true);
}
					
//==============================================================================

void CreateDiskImageDialog::updateData(const bool bToScreen)
{
	assert(gpoVolumeConfig);
	
	gpoDialogHelper->updateCreateDiskImageData(bToScreen);
	gpoDialogHelper->updateCryptData(bToScreen);
}

//==============================================================================

bool CreateDiskImageDialog::validate()
{
	if (!gpoDialogHelper->validateCreateDiskImageData())
		return false;
	if (!gpoDialogHelper->validateCryptData())
		return false;
		
	return true;
}

//==============================================================================
