/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "DialogHelper.h"
#include "QDebug.h"
#include "QConfig.h"
#include "eMountSys.h"
#include "eMountWin.h"
#include "QDialog.h"
#include "VolumeConfig.h"
#include "Util.h"
#include "PermDialog.h"
#include "MsgDialog.h"

#include "Pline/CryptData.h"
#include "Pline/CryptUtil.h"
#include "Pline/LoopUtil.h"
#include "Pline/MountData.h"
#include "Pline/MountUtil.h"

#include <glibmm/i18n.h>
#include <assert.h>

//==============================================================================

DialogHelper::DialogHelper(QDialog* poParent, VolumeConfig* poVolumeConfig)
{
	gpoParent = poParent;
	gpoVolumeConfig = poVolumeConfig;
	
	// crypt
	gpoEncryptedCheckButton = 0;
	gpoKeyFileFileChooser = 0;
	gpoUseKeyFileRadioButton = 0;
	gpoUsePasswordRadioButton = 0;

	gbCryptEnabled = false;

	gpoCipherComboBox = 0;
	gpoCBCComboBox = 0;
	gpoKeySizeComboBox = 0;
	
	gpstCipher = 0;
	gpstCBC = 0;
	gpstKeySize = 0;
	
	// mount
	gpoFSysComboBox = 0;
	gpvecFSys = 0;
	gpoMountPointFileChooser = 0;
	gpoPermToggleButton = 0;
	gbPermToggleButtonState = false;
	gpoPlaceAfterComboBox = 0;
	gpvecPlaceAfter = 0;
	gpoReadOnlyCheckButton = 0;
	gpoSourceFileChooser = 0;
	gpoVolumeLabelEntry = 0;
	gpoEmptyTrashCheckButton = 0;
	gpoEmptyTrashConfirmCheckButton = 0;
	
	// new disk image
	gpoSizeMRadioButton = 0;
	gpoSizeGRadioButton = 0;
	gpoVolumeSizeEntry = 0;
	gpullVolumeSize = 0;
	gpoMkFSysComboBox = 0;
	gpoSaveAsEntry = 0;	
	gpoAllocFullRadioButton = 0;
	gpoAllocCompactRadioButton = 0;
	gpbIsFullAlloc = 0;
}

//==============================================================================

DialogHelper::~DialogHelper()
{
}

//==============================================================================

void DialogHelper::buildCiphers()
{
	assert(gpoCipherComboBox);
	assert(gpoCBCComboBox);
	assert(gpoKeySizeComboBox);
	
	// clear combos
	gpoCipherComboBox->clear_items();
	gpoCBCComboBox->clear_items();
	gpoKeySizeComboBox->clear_items();
	
	// get data
	std::vector<Glib::ustring> vecCipher;
	std::vector<Glib::ustring> vecCBC;
	std::vector<Glib::ustring> vecKeySize;
	
	vecCipher.push_back(_("-- Auto"));
	vecCBC.push_back(_("-- Auto"));
	vecKeySize.push_back(_("-- Auto"));
	
	Pline::CryptUtil::getCipherList(&vecCipher);
	Pline::CryptUtil::getCBCList(&vecCBC);
	Pline::CryptUtil::getKeySizeList(&vecKeySize);
	
	// fill combos
	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi < vecCipher.size();
			++xi)
	{
		gpoCipherComboBox->append_text(vecCipher[xi]);
	}
	
	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi < vecCBC.size();
			++xi)
	{
		gpoCBCComboBox->append_text(vecCBC[xi]);
	}
	
	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi < vecKeySize.size();
			++xi)
	{
		gpoKeySizeComboBox->append_text(vecKeySize[xi]);
	}
}

//==============================================================================

void DialogHelper::buildPlaceAfter()
{
	assert(gpoPlaceAfterComboBox);
	assert(gpvecPlaceAfter);
	
	gpoPlaceAfterComboBox->clear_items();

	gpoPlaceAfterComboBox->append_text(_("-- Place First"));

	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi < gpvecPlaceAfter->size();
			++xi)
	{
		if ((*gpvecPlaceAfter)[xi] != gpoVolumeConfig->getData().getLabel())
			gpoPlaceAfterComboBox->append_text((*gpvecPlaceAfter)[xi]);
	}
}

//==============================================================================

Glib::ustring DialogHelper::createVolumeLabel(const Glib::ustring& stSrc)
{
	Glib::ustring stVolumeLabel = stSrc;
	size_t iPos;
	
	stVolumeLabel = Glib::path_get_basename(stSrc);

	if ((iPos = stVolumeLabel.rfind(".")) != std::string::npos)
		stVolumeLabel = stVolumeLabel.substr(0, iPos);
	
	if (stVolumeLabel.size() > 0)
		stVolumeLabel = stVolumeLabel.substr(0, 1).uppercase() + stVolumeLabel.substr(1);

	return stVolumeLabel;
}
								
//==============================================================================

void DialogHelper::onEncrypt()
{
	assert(gpoEncryptedCheckButton);
	
	if (gpoKeyFileFileChooser)
		gpoKeyFileFileChooser->set_sensitive(gpoEncryptedCheckButton->get_active());
	
	if (gpoUseKeyFileRadioButton)
	{
		gpoUseKeyFileRadioButton->set_sensitive(gpoEncryptedCheckButton->get_active());
		
		if (gpoKeyFileFileChooser && gpoEncryptedCheckButton->get_active())
			gpoKeyFileFileChooser->set_sensitive(gpoUseKeyFileRadioButton->get_active());
	}
	
	if (gpoUsePasswordRadioButton)
	{
		gpoUsePasswordRadioButton->set_sensitive(gpoEncryptedCheckButton->get_active());	
				
		if (gpoEncryptedCheckButton->get_active())
		{
			gpoUsePasswordRadioButton->set_active(true);
			
			if (gpoKeyFileFileChooser)
				gpoKeyFileFileChooser->set_sensitive(!gpoUsePasswordRadioButton->get_active());
		}
	}
	
	if (gpoCipherComboBox)
		gpoCipherComboBox->set_sensitive(gpoEncryptedCheckButton->get_active());
		
	if (gpoCBCComboBox)
		gpoCBCComboBox->set_sensitive(gpoEncryptedCheckButton->get_active());
		
	if (gpoKeySizeComboBox)
		gpoKeySizeComboBox->set_sensitive(gpoEncryptedCheckButton->get_active());
}

//==============================================================================

void DialogHelper::onFSysReadOnly()
{
	assert(gpoReadOnlyCheckButton);
	
	if (gpoPermToggleButton)
	{
		if (gpoReadOnlyCheckButton->get_active())
			setPermToggleButtonActive(false);				
					
		gpoPermToggleButton->set_sensitive(!gpoReadOnlyCheckButton->get_active());
	}
	
	if (gpoEmptyTrashCheckButton)
	{
		gpoEmptyTrashCheckButton->set_sensitive(!gpoReadOnlyCheckButton->get_active());
		gpoEmptyTrashCheckButton->set_active(false);
	}
	
	if (gpoEmptyTrashConfirmCheckButton)
	{
		gpoEmptyTrashConfirmCheckButton->set_sensitive(!gpoReadOnlyCheckButton->get_active());
		gpoEmptyTrashConfirmCheckButton->set_active(false);
	}
}

//==============================================================================

void DialogHelper::onPermissions()
{
	assert(gpoPermToggleButton);
	
	if (gbPermToggleButtonState)
		return;
		
	PermDialog *poPermDlg = 0;
	
	eMountSys::getInstance().getGuiBuilder()->get_widget_derived("PermDialog", poPermDlg);
	
	if (!poPermDlg)
		return;

	eMountWin::addOpenedDialog(poPermDlg);
	
	if (!gpoVolumeConfig->getMountData())
	{
		Pline::MountData *poMountData = 
			new Pline::MountData(gpoVolumeConfig->getData());
		
		gpoVolumeConfig->setMountData(poMountData);
		
		if (gpoMountPointFileChooser)
			poMountData->setDir(gpoMountPointFileChooser->get_current_folder());
	}
		
	poPermDlg->set_transient_for(*gpoParent);
	poPermDlg->setData(gpoVolumeConfig->getMountData());
	
	if (poPermDlg->run() == Gtk::RESPONSE_OK)
	{
		setPermToggleButtonActive(
			gpoVolumeConfig->getMountData()->getDirSetPerm());
		
		if (gpoReadOnlyCheckButton)
		{
			if (gpoVolumeConfig->getMountData()->getDirSetPerm())
				gpoReadOnlyCheckButton->set_active(false);
				
			gpoReadOnlyCheckButton->set_sensitive(!gpoVolumeConfig->getMountData()->getDirSetPerm());
		}
	}
	
	poPermDlg->hide();
}

//==============================================================================

void DialogHelper::onSaveAs()
{
	assert(gpoSaveAsEntry);
	
	Gtk::FileChooserDialog oDlg(*gpoParent, _("Save As"), Gtk::FILE_CHOOSER_ACTION_SAVE);
	
	oDlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	oDlg.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
	
	if (gpoSaveAsEntry->get_text().empty())
	{
		// set default path
		oDlg.set_current_folder(
			QConfig::getInstance().getDfDir());
	}
	else
	{
		oDlg.set_current_folder(
			Glib::path_get_dirname(gpoSaveAsEntry->get_text()));
	}

	if (oDlg.run() == Gtk::RESPONSE_OK)
	{
		gpoSaveAsEntry->set_text(oDlg.get_filename());
		gpoSaveAsEntry->set_position(oDlg.get_filename().size());
	}
	
	DialogHelper::onSource();
}

//==============================================================================

void DialogHelper::onSource()
{
	assert(gpoVolumeLabelEntry);
	
	assert(gpoSourceFileChooser || gpoSaveAsEntry);
		
	Glib::ustring stVolumeLabel;
	
	if (gpoSourceFileChooser)
	{		
		if (gstSourceFileChooserLastFilename.empty()
			&& !gpoVolumeConfig->getData().getLabel().empty())
		{
			stVolumeLabel = gpoVolumeConfig->getData().getLabel();
		}
		else
			stVolumeLabel = DialogHelper::createVolumeLabel(gpoSourceFileChooser->get_filename());
			
		gstSourceFileChooserLastFilename = gpoSourceFileChooser->get_filename();
	}
	else if (gpoSaveAsEntry)
		stVolumeLabel = DialogHelper::createVolumeLabel(gpoSaveAsEntry->get_text());
	
	gpoVolumeLabelEntry->set_text(stVolumeLabel);
	
}

//==============================================================================

void DialogHelper::onUseKeyFile()
{
	assert(gpoUseKeyFileRadioButton);
	
	if (gpoKeyFileFileChooser)
		gpoKeyFileFileChooser->set_sensitive(gpoUseKeyFileRadioButton->get_active());
}

//==============================================================================

void DialogHelper::onUsePassword()
{
	assert(gpoUsePasswordRadioButton);
	
	if (gpoKeyFileFileChooser)
		gpoKeyFileFileChooser->set_sensitive(false);
}

//==============================================================================

void DialogHelper::setPermToggleButtonActive(const bool bActive)
{
	assert(gpoPermToggleButton);
	
	gbPermToggleButtonState = true;
	gpoPermToggleButton->set_active(bActive);
	gbPermToggleButtonState = false;
}

//==============================================================================

void DialogHelper::updateCreateDiskImageData(const bool bToScreen)
{
	updateMountData(bToScreen);
	
	if (bToScreen)
	{
		if (gpoVolumeSizeEntry)
			gpoVolumeSizeEntry->set_text("");
			
		if (gpoSizeMRadioButton)
			gpoSizeMRadioButton->set_active();

		if (gpoMkFSysComboBox)
			gpoMkFSysComboBox->set_active_text(QConfig::getInstance().getDfFSys());
			
		if (gpoAllocFullRadioButton)
		{
			assert(gpbIsFullAlloc);
			
			gpoAllocFullRadioButton->set_active(true);
			*gpbIsFullAlloc = true;
		}
	}
	else
	{			
		// get size in bytes
		if (gpoVolumeSizeEntry)
		{
			assert(gpoSizeMRadioButton);
			assert(gpoSizeGRadioButton);

			Glib::ustring stVolumeSize = gpoVolumeSizeEntry->get_text();
			
			// check if numeric
			Glib::RefPtr<Glib::Regex> re = 
					Glib::Regex::create("^[0-9]+$");
			
			if (!re->match(stVolumeSize))
				*gpullVolumeSize = 0;
			else
			{
				unsigned long ulSize;
				unsigned long ulMpx = 0;

				ulSize = strtoul(stVolumeSize.c_str(), 0, 0);
			
				if (gpoSizeMRadioButton->get_active())
					ulMpx = 1024*1024;
				else if (gpoSizeGRadioButton->get_active())
					ulMpx = 1024*1024*1024;

				*gpullVolumeSize = (unsigned long long)ulSize * (unsigned long long)ulMpx;
			}
		}
		
		// allocation
		if (gpoAllocFullRadioButton)
		{
			assert(gpbIsFullAlloc);
			
			*gpbIsFullAlloc = gpoAllocFullRadioButton->get_active();
		}
		
		if (gpoMkFSysComboBox)
		{
			assert(gpoVolumeConfig->getMountData());
			
			gpoVolumeConfig->getMountData()->setFSys(gpoMkFSysComboBox->get_active_text());
			
			QConfig::getInstance().setDfFSys(gpoMkFSysComboBox->get_active_text());
		}
	}
}

//==============================================================================

void DialogHelper::updateCreateVolumeData(const bool bToScreen)
{
	updateMountData(bToScreen);
	updateCryptData(bToScreen);
	
	if (bToScreen)
	{
		gstLastDfSource = QConfig::getInstance().getDfSrc();
		
		if (gpoSourceFileChooser)
			gpoSourceFileChooser->set_current_folder("/dev");
		
		if (gpoMkFSysComboBox)
			gpoMkFSysComboBox->set_active_text(QConfig::getInstance().getDfFSys());
		
	}
	else
	{
		// set default path
		QConfig::getInstance().setDfSrc(gstLastDfSource);
			
		if (gpoMkFSysComboBox)
		{
			assert(gpoVolumeConfig->getMountData());
			
			gpoVolumeConfig->getMountData()->setFSys(gpoMkFSysComboBox->get_active_text());
			
			QConfig::getInstance().setDfFSys(gpoMkFSysComboBox->get_active_text());
		}
	}
}

//==============================================================================

void DialogHelper::updateCryptData(const bool bToScreen)
{
	if (bToScreen)
	{
		if (gpoCipherComboBox)
			gpoCipherComboBox->set_active(0);
		
		if (gpoCBCComboBox)
			gpoCBCComboBox->set_active(0);
		
		if (gpoKeySizeComboBox)
			gpoKeySizeComboBox->set_active(0);
			
		if (!gpoVolumeConfig->getCryptData())
		{
			if (gpoEncryptedCheckButton)
				gpoEncryptedCheckButton->set_active(false);
			
			if (gpoUseKeyFileRadioButton)
				gpoUseKeyFileRadioButton->set_sensitive(gbCryptEnabled);
			
			if (gpoUsePasswordRadioButton)
			{
				gpoUsePasswordRadioButton->set_active();
				gpoUsePasswordRadioButton->set_sensitive(gbCryptEnabled);
			}
					
			if (gpoKeyFileFileChooser)
			{
				gpoKeyFileFileChooser->set_sensitive(false);
				gpoKeyFileFileChooser->unselect_all();
				
				// set default path
				gpoKeyFileFileChooser->set_current_folder(
					QConfig::getInstance().getDfKey());
			}
			
			if (gpoCipherComboBox)
				gpoCipherComboBox->set_sensitive(gbCryptEnabled);
		
			if (gpoCBCComboBox)
				gpoCBCComboBox->set_sensitive(gbCryptEnabled);
		
			if (gpoKeySizeComboBox)
				gpoKeySizeComboBox->set_sensitive(gbCryptEnabled);
					
			return;
		}
		
		if (gpoEncryptedCheckButton)
			gpoEncryptedCheckButton->set_active(
				gpoVolumeConfig->getCryptData()->getIsEncrypted());
							
		if (gpoKeyFileFileChooser)
		{
			// set default path
			gpoKeyFileFileChooser->set_current_folder(
				QConfig::getInstance().getDfKey());	
				
			gpoKeyFileFileChooser->set_sensitive(gpoVolumeConfig->getCryptData()->getIsEncrypted());
			
			if (gpoVolumeConfig->getCryptData()->getIsEncrypted())
			{
				gpoKeyFileFileChooser->set_sensitive(
					(gpoVolumeConfig->getCryptData()->getKeyFile().empty() ? false : true));
			}
			
			if (!gpoVolumeConfig->getCryptData()->getKeyFile().empty())
				gpoKeyFileFileChooser->set_filename(gpoVolumeConfig->getCryptData()->getKeyFile());
		}
		
		if (gpoUseKeyFileRadioButton)
		{
			gpoUseKeyFileRadioButton->set_sensitive(gpoVolumeConfig->getCryptData()->getIsEncrypted());
			gpoUseKeyFileRadioButton->set_active(
				(gpoVolumeConfig->getCryptData()->getKeyFile().empty() ? false : true));
		}
			
		if (gpoUsePasswordRadioButton)
		{
			gpoUsePasswordRadioButton->set_sensitive(gpoVolumeConfig->getCryptData()->getIsEncrypted());
			gpoUsePasswordRadioButton->set_active(
				(gpoVolumeConfig->getCryptData()->getKeyFile().empty() ? true : false));
		}
		
		if (gpoCipherComboBox)
			gpoCipherComboBox->set_sensitive(gpoVolumeConfig->getCryptData()->getIsEncrypted());
		
		if (gpoCBCComboBox)
			gpoCBCComboBox->set_sensitive(gpoVolumeConfig->getCryptData()->getIsEncrypted());
		
		if (gpoKeySizeComboBox)
			gpoKeySizeComboBox->set_sensitive(gpoVolumeConfig->getCryptData()->getIsEncrypted());
			
	}
	else // update from screen to data
	{
		if (!gpoVolumeConfig->getCryptData())
		{
			if ((gpoEncryptedCheckButton 
				&& gpoEncryptedCheckButton->get_active())
				|| gbCryptEnabled)
			{
				Pline::CryptData *poCryptData = 
					new Pline::CryptData(gpoVolumeConfig->getData());
				
				gpoVolumeConfig->setCryptData(poCryptData);
				poCryptData->setIsEncrypted(true);
			}
		}
		
		if (!gpoVolumeConfig->getCryptData())
			return;
		
		if (gpoCipherComboBox)
		{
			assert(gpstCipher);
			
			*gpstCipher =
				(gpoCipherComboBox->get_active_row_number() == 0) ? "" : gpoCipherComboBox->get_active_text();
		}
		
		if (gpoCBCComboBox)
		{
			assert(gpstCBC);
			
			*gpstCBC = 
				(gpoCBCComboBox->get_active_row_number() == 0) ? "" : gpoCBCComboBox->get_active_text();
		}
		
		if (gpoKeySizeComboBox)
		{
			assert(gpstKeySize);
			
			*gpstKeySize = 
				(gpoKeySizeComboBox->get_active_row_number() == 0) ? "" : gpoKeySizeComboBox->get_active_text();
		}
		
		// delete cryptdata if necessary
		if (gpoEncryptedCheckButton && !gpoEncryptedCheckButton->get_active())
		{
			gpoVolumeConfig->removeCryptData();
			return;
		}
			
		if (gpoEncryptedCheckButton)
		{
			gpoVolumeConfig->getCryptData()->setIsEncrypted(gpoEncryptedCheckButton->get_active());
			
			if (!gpoEncryptedCheckButton->get_active())
				gpoVolumeConfig->getCryptData()->setKeyFile("");
		}
		
		if (gpoKeyFileFileChooser)
		{
			if (!gpoKeyFileFileChooser->get_filename().empty())
			{	
				bool bSetDfPath = false;
				
				if (gpoEncryptedCheckButton && !gpoEncryptedCheckButton->get_active())
					gpoVolumeConfig->getCryptData()->setKeyFile("");
				else if (	(gpoEncryptedCheckButton 
							&& gpoEncryptedCheckButton->get_active())
							|| gbCryptEnabled)
				{
					bSetDfPath = true;
					gpoVolumeConfig->getCryptData()->setKeyFile(gpoKeyFileFileChooser->get_filename());
				}
				
				if (bSetDfPath)
				{
					// set default path
					QConfig::getInstance().setDfKey(
						Glib::path_get_dirname(gpoKeyFileFileChooser->get_filename()));
				}
			}
		}
		
		if (gpoUsePasswordRadioButton && gpoUsePasswordRadioButton->get_active())
			gpoVolumeConfig->getCryptData()->setKeyFile("");
	}
}

//==============================================================================

void DialogHelper::updateEnlargeVolumeData(const bool bToScreen)
{
	updateCreateDiskImageData(bToScreen);
	
	if (bToScreen)
	{
		if (gpoSaveAsEntry)
			gpoSaveAsEntry->set_text(gpoVolumeConfig->getData().getSrc());
	}
}

//==============================================================================

void DialogHelper::updateInitVolumeData(const bool bToScreen)
{
	updateCreateVolumeData(bToScreen);
	
	if (bToScreen)
	{
		if (gpoSaveAsEntry)
			gpoSaveAsEntry->set_text(gpoVolumeConfig->getData().getSrc());
	}
}

//==============================================================================

void DialogHelper::updateMountData(const bool bToScreen)
{
	if (bToScreen)
	{
		if (!gpoVolumeConfig->getMountData())
		{
			if (gpoVolumeLabelEntry)
				gpoVolumeLabelEntry->set_text("");
				
			if (gpoSourceFileChooser)
			{
				gpoSourceFileChooser->unselect_all();
				
				// set default path
				gpoSourceFileChooser->set_current_folder(
					QConfig::getInstance().getDfSrc());
			}
			
			if (gpoSaveAsEntry)
				gpoSaveAsEntry->set_text("");
				
			if (gpoMountPointFileChooser)
			{
				gpoMountPointFileChooser->unselect_all();

				// set default path
				gpoMountPointFileChooser->set_current_folder(
					QConfig::getInstance().getDfDir());
			}
			
			if (gpoPermToggleButton)
				setPermToggleButtonActive(false);
				
			if (gpoReadOnlyCheckButton)
				gpoReadOnlyCheckButton->set_active(false);
				
			if (gpoFSysComboBox)
				gpoFSysComboBox->set_active(0);
			
			if (gpoPlaceAfterComboBox)
				gpoPlaceAfterComboBox->set_active(0);
				
			return;
		}

		if (gpoVolumeLabelEntry)
			gpoVolumeLabelEntry->set_text(gpoVolumeConfig->getData().getLabel());

		if (gpoSourceFileChooser)
		{
			if (Util::isFileExist(gpoVolumeConfig->getMountData()->getSrc()))
				gpoSourceFileChooser->set_filename(gpoVolumeConfig->getMountData()->getSrc());
			else
			{
				// set default path
				gpoSourceFileChooser->set_current_folder(
					QConfig::getInstance().getDfSrc());
			}
		}
		
		if (gpoMountPointFileChooser)
			gpoMountPointFileChooser->set_current_folder(gpoVolumeConfig->getMountData()->getDir());
		
		if (gpoPermToggleButton)
		{
			setPermToggleButtonActive(
				gpoVolumeConfig->getMountData()->getDirSetPerm());
		
			if (gpoReadOnlyCheckButton)
			{
				if (gpoVolumeConfig->getMountData()->getDirSetPerm())
					gpoReadOnlyCheckButton->set_active(false);
					
				gpoReadOnlyCheckButton->set_sensitive(!gpoVolumeConfig->getMountData()->getDirSetPerm());
			}
		}
		
		if (gpoReadOnlyCheckButton)
		{
			gpoReadOnlyCheckButton->set_active(
				gpoVolumeConfig->getMountData()->getReadOnly());
			
			if (gpoPermToggleButton)
			{
				if (gpoVolumeConfig->getMountData()->getReadOnly())
					setPermToggleButtonActive(false);
			
				gpoPermToggleButton->set_sensitive(!gpoVolumeConfig->getMountData()->getReadOnly());
			}
		}
		
		if (gpoFSysComboBox)
		{
			if (gpoVolumeConfig->getMountData()->getFSys().empty())
				gpoFSysComboBox->set_active(0);
			else
			{
				assert(gpvecFSys);
				
				if (Util::findSt(gpoVolumeConfig->getMountData()->getFSys(), *gpvecFSys))
				{
					gpoFSysComboBox->set_active_text(
						gpoVolumeConfig->getMountData()->getFSys());
				}
				else
				{
					gpoFSysComboBox->append_text(
						gpoVolumeConfig->getMountData()->getFSys());
					
					gpoFSysComboBox->set_active_text(
						gpoVolumeConfig->getMountData()->getFSys());
				}
			}
		}
		
		if (gpoPlaceAfterComboBox)
		{
			assert(gpvecPlaceAfter);

			if (gpoVolumeConfig->getData().getLabel().empty())
				gpoPlaceAfterComboBox->set_active(0);
			else
			{	
				int ci = 0;	
			
				for (   std::vector<Glib::ustring>::size_type xi = 0;
						xi < gpvecPlaceAfter->size();
						++xi)
				{
					if ((*gpvecPlaceAfter)[xi] == gpoVolumeConfig->getData().getLabel())
					{
						gpoPlaceAfterComboBox->set_active(ci);
						break;
					}
				
					++ci;
				}
			}
		}
	}
	else // update from screen to data
	{		
		Glib::ustring stOldVolumeLabel = gpoVolumeConfig->getData().getLabel();
		
		if (gpoVolumeLabelEntry)
			gpoVolumeConfig->getData().setLabel(gpoVolumeLabelEntry->get_text());
				
		if (gpoSourceFileChooser || gpoSaveAsEntry)
		{
			Glib::ustring stFileName =
				(gpoSourceFileChooser) ? gstSourceFileChooserLastFilename
						: gpoSaveAsEntry->get_text();
		
			if (!stFileName.empty())
			{
				gpoVolumeConfig->getData().setSrc(stFileName);
				
				// set default path
				QConfig::getInstance().setDfSrc(
					Glib::path_get_dirname(stFileName));
			}
		}
	
		if (!gpoVolumeConfig->getMountData())
		{
			Pline::MountData *poMountData = 
				new Pline::MountData(gpoVolumeConfig->getData());
		
			gpoVolumeConfig->setMountData(poMountData);
		}
				
		if (gpoMountPointFileChooser)
		{
			if (!gpoMountPointFileChooser->get_current_folder().empty())
			{
				gpoVolumeConfig->getMountData()->setDir(gpoMountPointFileChooser->get_current_folder());
				
				// set default path
				QConfig::getInstance().setDfDir(
					gpoMountPointFileChooser->get_current_folder());
			}
		}

		if (gpoReadOnlyCheckButton)
			gpoVolumeConfig->getMountData()->setReadOnly(gpoReadOnlyCheckButton->get_active());
		
		if (gpoFSysComboBox)
			gpoVolumeConfig->getMountData()->setFSys(
				(gpoFSysComboBox->get_active_row_number() == 0) ? "" : gpoFSysComboBox->get_active_text());
			
		// set new position
		if (gpoPlaceAfterComboBox)
		{
			assert(gpvecPlaceAfter);
			
			if (gpoPlaceAfterComboBox->get_active_row_number() == 0)
				gpoVolumeConfig->getData().setPos(1);
			else
			{
				Glib::ustring stPlaceAfter = gpoPlaceAfterComboBox->get_active_text();
					
				int ci = 0;	
				bool bSelf = false;
					
				for (   std::vector<Glib::ustring>::size_type xi = 0;
						xi < gpvecPlaceAfter->size();
						++xi)
				{
					if ((*gpvecPlaceAfter)[xi] == stPlaceAfter)
					{
						ci += (bSelf) ? 1 : 2;
						break;
					}
					else if ((*gpvecPlaceAfter)[xi] == stOldVolumeLabel)
						bSelf = true;
				
					++ci;
				}
					
				gpoVolumeConfig->getData().setPos(ci);
			}
		}
	}
}

//==============================================================================

bool DialogHelper::validateCreateDiskImageData()
{
	bool bErr = false;
	Glib::ustring stErr;
	Glib::ustring stField;
	
	assert(gpullVolumeSize);
	
	if (!validateMountData(true))
		return false;
	
	if (*gpullVolumeSize == ULONG_MAX || *gpullVolumeSize == 0)
	{
		stField = _("Volume Size");
		stErr = _("Invalid Size.");
		bErr = true;
		goto EXIT;
	}
	
EXIT:
	if (bErr)
	{
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
			
		tConfigItem.bAutoRun = true;
		tConfigItem.bWaitData = false;
		tConfigItem.poParent = gpoParent;
		tConfigItem.stMsg = Util::compose("%s: %s", stField.c_str(), stErr.c_str());
		
		MsgDialog oDlg(tConfigItem);
	}
	
	return !bErr;
}

//==============================================================================

bool DialogHelper::validateCryptData()
{
	bool bErr = false;
	Glib::ustring stErr;
	Glib::ustring stField;
	
	assert(gpoUseKeyFileRadioButton);
	
	if (gpoUseKeyFileRadioButton->get_active())
	{
		if (!Pline::CryptUtil::isValidKey(gpoVolumeConfig->getCryptData()->getKeyFile(), &stErr))
		{
			stField = _("Key File");
			bErr = true;
			goto EXIT;
		}
	}
	
EXIT:
	if (bErr)
	{
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
			
		tConfigItem.bAutoRun = true;
		tConfigItem.bWaitData = false;
		tConfigItem.poParent = gpoParent;
		tConfigItem.stMsg = Util::compose("%s: %s", stField.c_str(), stErr.c_str());
		
		MsgDialog oDlg(tConfigItem);
	}
	
	return !bErr;
}

//==============================================================================

bool DialogHelper::validateMountData(const bool bNewVolumeLabel)
{
	bool bErr = false;
	Glib::ustring stErr;
	Glib::ustring stField;
	
	assert(gpoVolumeConfig->getMountData());
	
	if (gpoSourceFileChooser)
	{
		if (!Pline::MountUtil::isValidSrc(gpoVolumeConfig->getData().getSrc())
			&& !Pline::LoopUtil::isValidSrc(gpoVolumeConfig->getData().getSrc()))
		{
			stErr = (gpoVolumeConfig->getData().getSrc().empty()) ? 
						_("Invalid source.")
							: Util::compose(_("%s Invalid."), gpoVolumeConfig->getData().getSrc().c_str());
							
			stField = (gpoSaveAsEntry) ? _("Save As") : _("Source");
			bErr = true;
			goto EXIT;
		}
	}

	if (gpoMountPointFileChooser)
	{
		if (!Pline::MountUtil::isValidDir(gpoVolumeConfig->getMountData()->getDir(), &stErr))
		{
			stField = _("Mount Point");
			bErr = true;
			goto EXIT;
		}
	}
	
	if (gpoVolumeLabelEntry)
	{
		if (!Pline::MountUtil::isValidLabel(gpoVolumeConfig->getData().getLabel(), &stErr))
		{
			stField = _("Volume Label");
			bErr = true;
			goto EXIT;
		}
		else
		{
			// Check unique lable
			if (gpvecPlaceAfter)
			{
				int ci = 0;
			
				for (   std::vector<Glib::ustring>::size_type xi = 0;
						xi < gpvecPlaceAfter->size();
						++xi)
				{
					if ((*gpvecPlaceAfter)[xi] == gpoVolumeConfig->getData().getLabel())
						ci++;
				}
			
				if ((bNewVolumeLabel && ci > 0)
					|| (!bNewVolumeLabel && ci > 1))
				{
					stField = _("Volume Label");
					stErr = _("Not a unique value.");
					bErr = true;
					goto EXIT;
				}
			}	
		}
	}
	
EXIT:
	if (bErr)
	{
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
			
		tConfigItem.bAutoRun = true;
		tConfigItem.bWaitData = false;
		tConfigItem.poParent = gpoParent;
		tConfigItem.stMsg = Util::compose("%s: %s", stField.c_str(), stErr.c_str());
		
		MsgDialog oDlg(tConfigItem);
	}
	
	return !bErr;
}

//==============================================================================
