/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DIALOGHELPER_H
#define DIALOGHELPER_H

#include <gtkmm.h>
#include <vector>

class QDialog;
class VolumeConfig;

class DialogHelper
{
public:
	DialogHelper(QDialog* poParent, VolumeConfig* poVolumeConfig);
	~DialogHelper();
	
	// crypt
	inline void setEnableEncryption(const bool b) {gbCryptEnabled = b;}
	inline void setEncryptedCheckButton(Gtk::CheckButton* po) {gpoEncryptedCheckButton = po;}
	inline void setKeyFileFileChooser(Gtk::FileChooserButton* po) {gpoKeyFileFileChooser = po;}
	inline void setUseKeyFileRadioButton(Gtk::RadioButton* po) {gpoUseKeyFileRadioButton = po;}
	inline void setUsePasswordRadioButton(Gtk::RadioButton* po) {gpoUsePasswordRadioButton = po;}
	
	inline void setCipherComboBox(
		Gtk::ComboBoxText* po,
		Glib::ustring* pst) {gpoCipherComboBox = po; gpstCipher = pst;}
		
	inline void setCBCComboBox(
		Gtk::ComboBoxText* po,
		Glib::ustring* pst) {gpoCBCComboBox = po; gpstCBC = pst;}
		
	inline void setKeySizeComboBox(
		Gtk::ComboBoxText* po,
		Glib::ustring* pst) {gpoKeySizeComboBox = po; gpstKeySize = pst;}
	
	// mount
	inline void setFSysComboBox(Gtk::ComboBoxEntryText* po) {gpoFSysComboBox = po;}
	inline void setFSysList(std::vector<Glib::ustring> *pvec) {gpvecFSys = pvec;}
	inline void setMountPointFileChooser(Gtk::FileChooserButton* po) {gpoMountPointFileChooser = po;}
	inline void setPermToggleButton(Gtk::ToggleButton* po) {gpoPermToggleButton = po;}
	inline void setPlaceAfterComboBox(Gtk::ComboBoxText* po) {gpoPlaceAfterComboBox = po;}
	inline void setPlaceAfterList(std::vector<Glib::ustring> *pvec) {gpvecPlaceAfter = pvec;}
	inline void setReadOnlyCheckButton(Gtk::CheckButton* po) {gpoReadOnlyCheckButton = po;}
	inline void setSourceFileChooser(Gtk::FileChooserButton* po) {gpoSourceFileChooser = po;}
	inline void setVolumeLabelEntry(Gtk::Entry* po) {gpoVolumeLabelEntry = po;}
	inline void setEmptyTrashCheckButton(Gtk::CheckButton* po) {gpoEmptyTrashCheckButton = po;}
	inline void setEmptyTrashConfirmCheckButton(Gtk::CheckButton* po) {gpoEmptyTrashConfirmCheckButton = po;}
	
	// create disk image
	inline void setMkFSysComboBox(Gtk::ComboBoxText* po) {gpoMkFSysComboBox = po;}
	inline void setSaveAsEntry(Gtk::Entry* po) {gpoSaveAsEntry = po;}
	
	inline void setAllocRadioButton(
		Gtk::RadioButton* poFull,
		Gtk::RadioButton* poCompact,
		bool *pbIsFull) {gpoAllocFullRadioButton = poFull; gpoAllocCompactRadioButton = poCompact; gpbIsFullAlloc = pbIsFull;}
	
	inline void setSizeRadioButton(
		Gtk::RadioButton* poMega,
		Gtk::RadioButton* poGiga) {gpoSizeMRadioButton = poMega; gpoSizeGRadioButton = poGiga;}
		
	inline void setVolumeSizeEntry(
		Gtk::Entry* po,
		unsigned long long *puiSize) {gpoVolumeSizeEntry = po; gpullVolumeSize = puiSize;}
	
	// crypt methods
	void buildCiphers();
	void onEncrypt();
	void onUseKeyFile();
	void onUsePassword();
	void updateCryptData(const bool bToScreen);
	bool validateCryptData();
	
	// mount methods
	void buildPlaceAfter();
	inline const Glib::ustring getSelSource() {return gstSourceFileChooserLastFilename;}
	void onFSysReadOnly();
	void onPermissions();
	void onSource();
	void updateMountData(const bool bToScreen);
	bool validateMountData(const bool bNewVolumeLabel);
	
	// create disk image methods
	void onSaveAs();
	void updateCreateDiskImageData(const bool bToScreen);
	void updateCreateVolumeData(const bool bToScreen);
	void updateEnlargeVolumeData(const bool bToScreen);
	void updateInitVolumeData(const bool bToScreen);
	
	bool validateCreateDiskImageData();
	
protected:

private:
	QDialog* gpoParent;
	VolumeConfig* gpoVolumeConfig;
	
	// new disk image
	Gtk::RadioButton* gpoSizeMRadioButton;
	Gtk::RadioButton* gpoSizeGRadioButton;
	Gtk::Entry* gpoVolumeSizeEntry;
	unsigned long long *gpullVolumeSize;
	Gtk::ComboBoxText* gpoMkFSysComboBox;
	Gtk::Entry* gpoSaveAsEntry;
	Gtk::RadioButton* gpoAllocFullRadioButton;
	Gtk::RadioButton* gpoAllocCompactRadioButton;
	bool *gpbIsFullAlloc;
	
	// new volume
	Glib::ustring gstLastDfSource;
	
	// mount
	std::vector<Glib::ustring>* gpvecFSys;
	Gtk::ComboBoxEntryText* gpoFSysComboBox;
	Gtk::FileChooserButton* gpoMountPointFileChooser;
	Gtk::ToggleButton* gpoPermToggleButton;
	bool gbPermToggleButtonState; // signla_clicked works same as signal_toggled, fixed
	Gtk::ComboBoxText* gpoPlaceAfterComboBox;
	std::vector<Glib::ustring>* gpvecPlaceAfter;
	Gtk::CheckButton* gpoReadOnlyCheckButton;
	Gtk::FileChooserButton* gpoSourceFileChooser;
	Glib::ustring gstSourceFileChooserLastFilename; // source can be messed up, this fixes it
	Gtk::RadioButton* gpoUseKeyFileRadioButton;
	Gtk::RadioButton* gpoUsePasswordRadioButton;
	Gtk::Entry* gpoVolumeLabelEntry;
	Gtk::CheckButton* gpoEmptyTrashCheckButton;
	Gtk::CheckButton* gpoEmptyTrashConfirmCheckButton;
	
	// crypt
	Gtk::CheckButton* gpoEncryptedCheckButton;
	Gtk::FileChooserButton* gpoKeyFileFileChooser;
	
	bool gbCryptEnabled;
	
	Gtk::ComboBoxText* gpoCipherComboBox;
	Gtk::ComboBoxText* gpoCBCComboBox;
	Gtk::ComboBoxText* gpoKeySizeComboBox;
	
	Glib::ustring* gpstCipher;
	Glib::ustring* gpstCBC;
	Glib::ustring* gpstKeySize;
	
	void setPermToggleButtonActive(const bool bActive);
	
	static Glib::ustring createVolumeLabel(const Glib::ustring& stSrc);
};
	
#endif /* DIALOGHELPER_H */
