/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Icon.h"
#include "QDebug.h"
#include "eMountSys.h"

#include "Gimp/emount.c"
#include "Gimp/media-eject-normal.c"
#include "Gimp/media-mount-disabled.c"
#include "Gimp/media-mount-normal.c"
#include "Gimp/process-stop-disabled.c"
#include "Gimp/process-stop-normal.c"

#include <iostream>
#include <glibmm/i18n.h>

//==============================================================================

Icon::Icon()
{	
	// init
	gbErrorMsgShown = false;
	gpoBtnCancelDisabledIcon = 0;
	gpoBtnCancelNormalIcon = 0;
	gpoBtnEjectNormalIcon = 0;
	gpoBtnMountDisabledIcon = 0;
	gpoBtnMountNormalIcon = 0;
			
	// draw icons
	gpoBtnCancelDisabledIcon = drawIcon("", 
										process_stop_disabled_gimp.pixel_data, 
										process_stop_disabled_gimp.width,
										process_stop_disabled_gimp.height,
										22, 
										22);
										
	gpoBtnCancelNormalIcon = drawIcon(  "", 
										process_stop_normal_gimp.pixel_data, 
										process_stop_normal_gimp.width,
										process_stop_normal_gimp.height,
										22, 
										22);

	gpoBtnEjectNormalIcon = drawIcon(  "", 
										media_eject_normal_gimp.pixel_data, 
										media_eject_normal_gimp.width,
										media_eject_normal_gimp.height,
										22, 
										22);
	
	gpoBtnMountDisabledIcon = drawIcon( "", 
										media_mount_disabled_gimp.pixel_data, 
										media_mount_disabled_gimp.width,
										media_mount_disabled_gimp.height,
										22, 
										22);
	
	gpoBtnMountNormalIcon = drawIcon(   "", 
										media_mount_normal_gimp.pixel_data, 
										media_mount_normal_gimp.width,
										media_mount_normal_gimp.height,
										22, 
										22);

	gpoSysIcon16 = drawIcon("", 
							emount_icon_gimp.pixel_data, 
							emount_icon_gimp.width, 
							emount_icon_gimp.height, 
							16, 
							16);
	
	gpoSysIcon32 = drawIcon("", 
							emount_icon_gimp.pixel_data, 
							emount_icon_gimp.width, 
							emount_icon_gimp.height, 
							32, 
							32);
	
	gpoSysIcon64 = drawIcon("", 
							emount_icon_gimp.pixel_data, 
							emount_icon_gimp.width, 
							emount_icon_gimp.height, 
							64, 
							64);
}

//==============================================================================

Icon::~Icon()
{
	if (gpoBtnCancelDisabledIcon)
		delete gpoBtnCancelDisabledIcon;
	
	if (gpoBtnCancelNormalIcon)
		delete gpoBtnCancelNormalIcon;
	
	if (gpoBtnEjectNormalIcon)
		delete gpoBtnEjectNormalIcon;
	
	if (gpoBtnMountDisabledIcon)
		delete gpoBtnMountDisabledIcon;
	
	if (gpoBtnMountNormalIcon)
		delete gpoBtnMountNormalIcon;
	
	if (gpoSysIcon16)
		delete gpoSysIcon16;
	
	if (gpoSysIcon64)
		delete gpoSysIcon64;
}

//==============================================================================

Icon& Icon::getInstance()
{
	static Icon oIcon;
	
	return oIcon;
}

//==============================================================================

Glib::RefPtr<Gdk::Pixbuf>* Icon::drawIcon(  std::string stFilename,
											const guint8 *pixel_data,
											const int data_width,
											const int data_height,
											const int iWidth, 
											const int iHeight)
{
	try
	{
		Glib::RefPtr<Gdk::Pixbuf> *poPix = new Glib::RefPtr<Gdk::Pixbuf>;		
		Glib::RefPtr<Gdk::Pixbuf> oPix;
	
		oPix = 
			(pixel_data) ? Gdk::Pixbuf::create_from_data(   pixel_data, 
															Gdk::COLORSPACE_RGB, 
															true, 
															8, // bit per pixel
															data_width, 
															data_height,
															data_width*4) // 4: RGBA
			: Gdk::Pixbuf::Pixbuf::create_from_file(stFilename);
			
		oPix = 	oPix->scale_simple(iWidth, iHeight, Gdk::INTERP_BILINEAR);
				
		*poPix = oPix->copy();
		
		return poPix;
	}
	catch (const Glib::FileError & ex)
	{
		std::cerr << ex.what() << std::endl;
		
		if (eMountSys::getInstance().getRunMode() == eMountSys::kRunGui
			&& !gbErrorMsgShown)
		{
			Gtk::MessageDialog oDialog(
				_("Failed loading icons."),
				false,
				Gtk::MESSAGE_ERROR,
				Gtk::BUTTONS_OK,
				true
			);
			
			oDialog.run();
			
			gbErrorMsgShown = true;
		}
		
		return 0;
	}
}

//==============================================================================
