/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "EnlargeVolumeFilter.h"
#include "../QDebug.h"
#include "../Util.h"

#include "FSysUtil.h"

#include <glibmm/i18n.h>
#include <assert.h>

namespace Pline
{

//==============================================================================

EnlargeVolumeFilter::EnlargeVolumeFilter()
: Filter()
{
	gpoMountData = 0;
}

//==============================================================================

EnlargeVolumeFilter::~EnlargeVolumeFilter()
{
}

//==============================================================================

bool EnlargeVolumeFilter::run()
{
	assert(gpoMountData);
	
	// reset error
	gstLastError.clear();
	
	if (!Util::isDevice(gpoMountData->getSrc(), &gstLastError))
		return false;
	
	if (!Util::isDirectory(gpoMountData->getDir(), &gstLastError))
		return false;
		
	if (gpoMountData->getFSys().empty())
	{
		gstLastError = _("Invalid Filesystem.");
		return false;
	}
	
	return FSysUtil::enlarge(	gpoMountData->getSrc(),
								gpoMountData->getDir(),
								gpoMountData->getFSys(),
								&gstLastError);
}

//==============================================================================

void EnlargeVolumeFilter::setSrcData(MountData *poData)
{
	gpoMountData = poData;
	
	Filter::setSrcData(poData);
}

//==============================================================================

} // Pline
