/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ProcessBasePline.h"
#include "QDebug.h"
#include "ProcessBaseFilter.h"

namespace Pline
{

//==============================================================================

ProcessBasePline::ProcessBasePline(	const eDataMode eDataModeItem,
									VolumeConfig *poVolumeConfig)
: Pline(eDataModeItem, poVolumeConfig)
{
	gpoRunningFilter = 0;
	gbCanCancel = false;
}

//==============================================================================

ProcessBasePline::~ProcessBasePline()
{
}

//==============================================================================

bool ProcessBasePline::cancel()
{	
	if (gpoRunningFilter && gbCanCancel)
	{
		gtLastError.bHasCancelledAction = true;
		
		return gpoRunningFilter->cancel();
	}
	else
		return false;
}

//==============================================================================

void ProcessBasePline::onCanCancelChanged(bool b)
{
	gbCanCancel = b;
}

//==============================================================================

bool ProcessBasePline::onFilterPreRun(Filter *poFilter)
{
	gbCanCancel = false;
	
	(poFilter->getFilterType() == "Process") ?
		gpoRunningFilter = (ProcessBaseFilter*)poFilter
			: gpoRunningFilter = 0;
	
	if (gpoRunningFilter)
	{
		gpoRunningFilter->signal_cancancel_changed().connect(
			sigc::mem_fun(*this, &ProcessBasePline::onCanCancelChanged));
	
		gbCanCancel = gpoRunningFilter->isCanCancel();
	}
	
	return true;
}

//==============================================================================


} // Pline	
