/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "PropDialog.h"
#include "QDebug.h"
#include "QConfig.h"
#include "VolumeConfig.h"
#include "DialogHelper.h"
#include "Util.h"
#include "Pline/MountUtil.h"
#include "Pline/CloneData.h"

#include <glibmm/i18n.h>
#include <assert.h>

//==============================================================================

PropDialog::PropDialog( BaseObjectType* cobject,
						const Glib::RefPtr<Gtk::Builder>& refGlade)
: QDialog(cobject)
{
	set_modal(true);
	set_title(_("Volume Properties"));
	
	gpoDialogHelper = 0;
	gpoVolumeConfig = 0;
	gpoSourceFileChooser = 0;
	gpoKeyFileFileChooser = 0;
	gpvecVolumeConfigLabel = 0;
	gpvecPlineStatus = 0;
	
	UtilGui::mapPropDlg(refGlade, gtPropDlg);	
		
	gtPropDlg.poApplyButton->signal_clicked().connect(
		sigc::mem_fun(*this, &PropDialog::onApply));
		
	gtPropDlg.poCloseButton->signal_clicked().connect(
		sigc::mem_fun(*this, &PropDialog::onClose));
	
	gtPropDlg.tMountItem.poPermToggleButton->signal_released().connect(
		sigc::mem_fun(*this, &PropDialog::onPermissions));
		
	gtPropDlg.tMountItem.poReadOnlyCheckButton->signal_toggled().connect(
		sigc::mem_fun(*this, &PropDialog::onReadOnly));
		
	gtPropDlg.tMountItem.poEncryptedCheckButton->signal_toggled().connect(
		sigc::mem_fun(*this, &PropDialog::onEncrypt));
	
	gtPropDlg.tMountItem.poUsePasswordRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &PropDialog::onUsePassword));
	
	gtPropDlg.tMountItem.poUseKeyFileRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &PropDialog::onUseKeyFile));
	
	gtPropDlg.tCloneItem.poDestinationButton->signal_clicked().connect(
		sigc::mem_fun(*this, &PropDialog::onCloneBrowse));
	
	gtPropDlg.tOptionsItem.poEmptyTrashCheckButton->signal_toggled().connect(
		sigc::mem_fun(*this, &PropDialog::onOptEmptyTrash));
		
	Gtk::ComboBoxEntryText* pocmbFSys = manage(new Gtk::ComboBoxEntryText);
			
	gtPropDlg.tMountItem.poFSysVBox->pack_start(*pocmbFSys, false, false);
	gtPropDlg.tMountItem.poFSysComboBox = pocmbFSys;
	
	Gtk::ComboBoxText* pocmbPlaceAfter = manage(new Gtk::ComboBoxText);
			
	gtPropDlg.tMountItem.poPlaceAfterVBox->pack_start(*pocmbPlaceAfter, false, false);
	gtPropDlg.tMountItem.poPlaceAfterComboBox = pocmbPlaceAfter;
	
	show_all_children();
}

//==============================================================================

PropDialog::~PropDialog()
{	
	if (gpoDialogHelper)
		delete gpoDialogHelper;
		
	if (gpoSourceFileChooser)
		delete gpoSourceFileChooser;
		
	if (gpoKeyFileFileChooser)
		delete gpoKeyFileFileChooser;
}

//==============================================================================

void PropDialog::buildFSys()
{
	unsigned int xui;

	gvecFSys.clear();
	
	gtPropDlg.tMountItem.poFSysComboBox->clear_items();
	gtPropDlg.tMountItem.poFSysComboBox->append_text(_("-- Auto"));
	
	Pline::MountUtil::getFSysList(&gvecFSys);

	for (xui=0; xui<gvecFSys.size(); ++xui)
		gtPropDlg.tMountItem.poFSysComboBox->append_text(gvecFSys[xui]);
}

//==============================================================================

void PropDialog::onApply()
{
	updateData(false);
	
	if (!validate())
		return;
		
	QDialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void PropDialog::onCloneBrowse()
{
	Gtk::FileChooserDialog oDlg(*this, _("Save As"), Gtk::FILE_CHOOSER_ACTION_SAVE);
	
	oDlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	oDlg.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
	
	if (gtPropDlg.tCloneItem.poDestinationEntry->get_text().empty())
	{
		// set default path
		oDlg.set_current_folder(
			QConfig::getInstance().getDfCloneDir());
	}
	else
	{
		oDlg.set_current_folder(
			Glib::path_get_dirname(gtPropDlg.tCloneItem.poDestinationEntry->get_text()));
	}

	if (oDlg.run() == Gtk::RESPONSE_OK)
	{
		gtPropDlg.tCloneItem.poDestinationEntry->set_text(
			oDlg.get_filename());
			
		gtPropDlg.tCloneItem.poDestinationEntry->set_position(
			oDlg.get_filename().size());
	}
}

//==============================================================================

void PropDialog::onClose()
{
	QDialog::response(Gtk::RESPONSE_CANCEL);
}

//==============================================================================

void PropDialog::onEncrypt()
{
	gpoDialogHelper->onEncrypt();
}

//==============================================================================

void PropDialog::onOptEmptyTrash()
{
	if (gtPropDlg.tOptionsItem.poEmptyTrashCheckButton->get_active())
	{
		gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_active(true);
		gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_sensitive(true);
	}
	else
	{
		gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_active(false);
		gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_sensitive(false);
	}
}

//==============================================================================

void PropDialog::onPermissions()
{
	gpoDialogHelper->onPermissions();
}

//==============================================================================

void PropDialog::onReadOnly()
{
	gpoDialogHelper->onFSysReadOnly();
}

//==============================================================================

void PropDialog::onSource()
{
	gpoDialogHelper->onSource();
}

//==============================================================================

void PropDialog::onUseKeyFile()
{
	gpoDialogHelper->onUseKeyFile();
}

//==============================================================================

void PropDialog::onUsePassword()
{
	gpoDialogHelper->onUsePassword();
}

//==============================================================================

void PropDialog::setData(   VolumeConfig *poVolumeConfig,
							std::vector<Glib::ustring> *pvecVolumeConfigLabel,
							std::vector<Pline::MountPline::tDumpStatus> *pvecPlineStatus,
							const Glib::ustring& stFSys,
							const Glib::ustring& stCipher,
							const Glib::ustring& stDiskSize,
							const Glib::ustring& stDiskUsedSpace,
							const Glib::ustring& stDiskAvailableSpace)
{
	gpoVolumeConfig = poVolumeConfig;
	gpvecVolumeConfigLabel = pvecVolumeConfigLabel;
	
	gpvecPlineStatus = pvecPlineStatus;
	gstFSys = stFSys;
	gstCipher = stCipher;
	gstDiskSize = stDiskSize;
	gstDiskUsedSpace = stDiskUsedSpace;
	gstDiskAvailableSpace = stDiskAvailableSpace;
	
	buildFSys();

	// work around for bug in FileChooserButton (on cancel the control would be messed up)
	if (gpoSourceFileChooser)
		delete gpoSourceFileChooser;
	
	gpoSourceFileChooser = new Gtk::FileChooserButton;
	
	gtPropDlg.tMountItem.poSourceVBox->pack_start(*gpoSourceFileChooser, false, false);
	gtPropDlg.tMountItem.poSourceFileChooser = gpoSourceFileChooser;
	
	gtPropDlg.tMountItem.poSourceFileChooser->signal_selection_changed().connect(
		sigc::mem_fun(*this, &PropDialog::onSource));
		
	if (gpoKeyFileFileChooser)
		delete gpoKeyFileFileChooser;
	
	gpoKeyFileFileChooser = new Gtk::FileChooserButton;
	
	gtPropDlg.tMountItem.poKeyFileVBox->pack_start(*gpoKeyFileFileChooser, false, false);
	gtPropDlg.tMountItem.poKeyFileFileChooser = gpoKeyFileFileChooser;
		
	show_all_children();
	// end of work around
	
	//
	// Dialog Helper
	//
	
	if (gpoDialogHelper)
		delete gpoDialogHelper;
		
	gpoDialogHelper = new DialogHelper(this, gpoVolumeConfig);
	
	// mount
	gpoDialogHelper->setFSysComboBox(gtPropDlg.tMountItem.poFSysComboBox);
	gpoDialogHelper->setFSysList(&gvecFSys);
	gpoDialogHelper->setMountPointFileChooser(gtPropDlg.tMountItem.poMountPointFileChooser);
	gpoDialogHelper->setPermToggleButton(gtPropDlg.tMountItem.poPermToggleButton);
	gpoDialogHelper->setPlaceAfterComboBox(gtPropDlg.tMountItem.poPlaceAfterComboBox);
	gpoDialogHelper->setPlaceAfterList(gpvecVolumeConfigLabel);
	gpoDialogHelper->setReadOnlyCheckButton(gtPropDlg.tMountItem.poReadOnlyCheckButton);
	gpoDialogHelper->setSourceFileChooser(gtPropDlg.tMountItem.poSourceFileChooser);
	gpoDialogHelper->setVolumeLabelEntry(gtPropDlg.tMountItem.poVolumeLabelEntry);
	gpoDialogHelper->setEmptyTrashCheckButton(gtPropDlg.tOptionsItem.poEmptyTrashCheckButton);
	gpoDialogHelper->setEmptyTrashConfirmCheckButton(gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton);
	
	// crypt
	gpoDialogHelper->setEncryptedCheckButton(gtPropDlg.tMountItem.poEncryptedCheckButton);
	gpoDialogHelper->setKeyFileFileChooser(gtPropDlg.tMountItem.poKeyFileFileChooser);
	gpoDialogHelper->setUseKeyFileRadioButton(gtPropDlg.tMountItem.poUseKeyFileRadioButton);
	gpoDialogHelper->setUsePasswordRadioButton(gtPropDlg.tMountItem.poUsePasswordRadioButton);
	
	gpoDialogHelper->buildPlaceAfter();
	
	updateData(true);
}

//==============================================================================

void PropDialog::updateData(const bool bToScreen)
{
	assert(gpoVolumeConfig);
	assert(gpoSourceFileChooser);
	assert(gpoKeyFileFileChooser);
	
	if (bToScreen)
	{
		Glib::ustring stSrc;
		Glib::ustring stMountPoint;
		
		for (   std::vector<Pline::MountPline::tDumpStatus>::size_type xi = 0;
				xi < gpvecPlineStatus->size();
				++xi)
		{
			Pline::MountPline::tDumpStatus tDumpStatusItem = (*gpvecPlineStatus)[xi];
			
			if (!tDumpStatusItem.stSrc.empty() && stSrc.empty())
				stSrc = tDumpStatusItem.stSrc;
			
			if (tDumpStatusItem.stSrcType == "Directory" && stMountPoint.empty())
				stMountPoint = tDumpStatusItem.stSrc;
			
			if (!stSrc.empty() && !stMountPoint.empty())
				break;
		}
		
		// Status
		gtPropDlg.tStatusItem.poVolumeLabelEntry->set_text(gpoVolumeConfig->getData().getLabel());
		gtPropDlg.tStatusItem.poSourceEntry->set_text(stSrc);
		gtPropDlg.tStatusItem.poSourceEntry->set_position(stSrc.size());
		
		if (gpoVolumeConfig->getMountData())
		{
			gtPropDlg.tStatusItem.poMountPointEntry->set_text(stMountPoint);
			gtPropDlg.tStatusItem.poMountPointEntry->set_position(stMountPoint.size());
			gtPropDlg.tStatusItem.poFSysEntry->set_text(
				Util::compose( 	"%s (%s)",
								(gpoVolumeConfig->getMountData()->getFSys().empty()) ? _("Auto") : gpoVolumeConfig->getMountData()->getFSys().c_str(),
								(gstFSys.empty() ? _("Auto") : gstFSys.c_str())));
			
			updateExtraStatus();
		}
		else
		{
			gtPropDlg.tStatusItem.poMountPointEntry->set_text("");
			gtPropDlg.tStatusItem.poFSysEntry->set_text("");
		}
						
		(gstDiskSize.empty()) ?
			gtPropDlg.tStatusItem.poDiskCapacityEntry->set_text(_("n/a")) :
			gtPropDlg.tStatusItem.poDiskCapacityEntry->set_text(gstDiskSize);
		(gstDiskUsedSpace.empty()) ?
			gtPropDlg.tStatusItem.poDiskUsedEntry->set_text(_("n/a")) :
			gtPropDlg.tStatusItem.poDiskUsedEntry->set_text(gstDiskUsedSpace);
		(gstDiskAvailableSpace.empty()) ?
			gtPropDlg.tStatusItem.poDiskFreeEntry->set_text(_("n/a")) :
			gtPropDlg.tStatusItem.poDiskFreeEntry->set_text(gstDiskAvailableSpace);
		
		// DialogHelper
		gpoDialogHelper->updateMountData(true);
		gpoDialogHelper->updateCryptData(true);
			
		// Clone
		if (gpoVolumeConfig->getCloneData())
		{
			gtPropDlg.tCloneItem.poDestinationEntry->set_text(
				gpoVolumeConfig->getCloneData()->getDest());

			gtPropDlg.tCloneItem.poDestinationEntry->set_position(
				gpoVolumeConfig->getCloneData()->getDest().size());

			gtPropDlg.tCloneItem.poOverwriteCheckButton->set_active(
				!gpoVolumeConfig->getCloneData()->getCanOverwrite());			
		}
		else
		{
			gtPropDlg.tCloneItem.poDestinationEntry->set_text("");
			gtPropDlg.tCloneItem.poOverwriteCheckButton->set_active(true);
		}
		
		// Option
		gtPropDlg.tOptionsItem.poAutomountCheckButton->set_active(gpoVolumeConfig->isAutomount());
		
		if (gpoVolumeConfig->getMountData())
		{
			if (gpoVolumeConfig->getMountData()->getReadOnly())
			{
				gtPropDlg.tOptionsItem.poEmptyTrashCheckButton->set_active(false);
				gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_active(false);
		
				gtPropDlg.tOptionsItem.poEmptyTrashCheckButton->set_sensitive(false);
				gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_sensitive(false);
			}
			else
			{
				gtPropDlg.tOptionsItem.poEmptyTrashCheckButton->set_active(gpoVolumeConfig->getMountData()->isEmptyTrash());
				gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_active(gpoVolumeConfig->getMountData()->isEmptyTrashConfirm());
		
				if (gpoVolumeConfig->getMountData()->isEmptyTrash())
				{
					gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_sensitive(true);
					gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_active(gpoVolumeConfig->getMountData()->isEmptyTrashConfirm());
				}
				else
				{
					gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_sensitive(false);
					gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->set_active(false);
				}
			}
		}
	}
	else
	{
		// DialogHelper
		gpoDialogHelper->updateMountData(false);
		gpoDialogHelper->updateCryptData(false);
		
		// Clone
		if (gtPropDlg.tCloneItem.poDestinationEntry->get_text().empty())
		{
			if (gpoVolumeConfig->getCloneData())
				gpoVolumeConfig->removeCloneData();
		}
		else
		{
			if (!gpoVolumeConfig->getCloneData())
			{
				Pline::CloneData *poCloneData = 
					new Pline::CloneData(gpoVolumeConfig->getData());
				
				gpoVolumeConfig->setCloneData(poCloneData);
			}
		}
		
		if (gpoVolumeConfig->getCloneData())
		{
			gpoVolumeConfig->getCloneData()->setDest(
				gtPropDlg.tCloneItem.poDestinationEntry->get_text());
			
			gpoVolumeConfig->getCloneData()->setCanOverwrite(
				!gtPropDlg.tCloneItem.poOverwriteCheckButton->get_active());
			
			// set default path
			QConfig::getInstance().setDfCloneDir(
				Glib::path_get_dirname(gtPropDlg.tCloneItem.poDestinationEntry->get_text()));
		}
		
		// Option
		gpoVolumeConfig->setAutomount(gtPropDlg.tOptionsItem.poAutomountCheckButton->get_active());
		
		if (gpoVolumeConfig->getMountData())
		{
			gpoVolumeConfig->getMountData()->setEmptyTrash(gtPropDlg.tOptionsItem.poEmptyTrashCheckButton->get_active());
			gpoVolumeConfig->getMountData()->setEmptyTrashConfirm(gtPropDlg.tOptionsItem.poEmptyTrashConfirmCheckButton->get_active());
		}
	}
}

//==============================================================================

void PropDialog::updateExtraStatus()
{
	Glib::ustring stExtra;
	
	for (	std::vector<Pline::MountPline::tDumpStatus>::size_type xi = 0;
			xi < gpvecPlineStatus->size();
			++xi)
	{
		Pline::MountPline::tDumpStatus *ptDumpStatusItem =
			&(*gpvecPlineStatus)[xi];
		
		if (ptDumpStatusItem->stSrc.empty())
			continue;
			
		if (ptDumpStatusItem->stFilterName == "Crypt" && !gstCipher.empty())
		{
			stExtra += Util::compose(	"%s %s %s. %s\n",
										ptDumpStatusItem->stFilterName.c_str(),
										ptDumpStatusItem->stSrcType.c_str(),
										ptDumpStatusItem->stSrc.c_str(),
										gstCipher.c_str());
		}
		else
			stExtra += Util::compose(	"%s %s %s\n",
										ptDumpStatusItem->stFilterName.c_str(),
										ptDumpStatusItem->stSrcType.c_str(),
										ptDumpStatusItem->stSrc.c_str());
	}
		
	gtPropDlg.tStatusItem.poExtraTextView->get_buffer()->set_text(stExtra);
}

//==============================================================================

bool PropDialog::validate()
{
	if (!gpoDialogHelper->validateMountData(false))
		return false;
	
	if (gpoVolumeConfig->getCryptData())
	{
		if (!gpoDialogHelper->validateCryptData())
			return false;
	}
	
	return true;
}

//==============================================================================
