/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ThreadVolumeStatus.h"
#include "QDebug.h"
#include "VolumeMountGui.h"

#include <assert.h>

#ifdef QDEBUG
	#define TIMER_UPDATE_SEC 5
#else
	#define TIMER_UPDATE_SEC 45
#endif

//==============================================================================

ThreadVolumeStatus::ThreadVolumeStatus( std::list<VolumeGui*>& lstVolumeGui,
										Glib::Mutex& omtxVolumeGuiLst)
: glstVolumeGui(lstVolumeGui),
  gmtxVolumeGuiLst(omtxVolumeGuiLst)
{
	gpoThread = 0;
	
	gbExit = false;
	gbUpdateStatus = false;
	
	// dispatcher
	goDispatcher.connect(sigc::mem_fun(*this, &ThreadVolumeStatus::onDispatcher));
}

//==============================================================================

ThreadVolumeStatus::~ThreadVolumeStatus()
{
	g_return_if_fail(!gpoThread);
}

//==============================================================================

void ThreadVolumeStatus::join()
{
	if (goTimeoutFunConn)
		goTimeoutFunConn.disconnect();
		
	gmtxTimer.lock();
		
	gbExit = true;
	gcondTimer.signal();
	
	gmtxTimer.unlock();
		
	gpoThread->join();
	gpoThread = NULL;
	
	debug_message(DEBUG_ALL, "ThreadVolumeStatus completed.");
}

//==============================================================================

void ThreadVolumeStatus::launch()
{
	assert(!gpoThread);
		
	// Create a joinable thread.
	gpoThread = Glib::Thread::create(sigc::mem_fun(*this, &ThreadVolumeStatus::threadFun), true);
	
	goTimeoutFunConn =
			Glib::signal_timeout().connect_seconds(
				sigc::mem_fun(*this, &ThreadVolumeStatus::onTimer),
			    TIMER_UPDATE_SEC);
}

//==============================================================================

void ThreadVolumeStatus::onDispatcher()
{
	Glib::Mutex::Lock lock(gmtxVolumeGuiLst);

	for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
			ite != glstVolumeGui.end();
			++ite)
	{
		VolumeMountGui* poVolumeGui = (VolumeMountGui*)*ite;
		
		if (poVolumeGui->lockThread())
		{
			poVolumeGui->updateVolumeStatus();
			poVolumeGui->lockThread(false);
		}
	}
}

//==============================================================================

bool ThreadVolumeStatus::onTimer()
{
	assert(gpoThread);
	
	Glib::Mutex::Lock lock(gmtxTimer);
		
	gbUpdateStatus = true;
	gcondTimer.signal();
			
	return true;
}

//==============================================================================

void ThreadVolumeStatus::threadFun()
{
	debug_message(DEBUG_ALL, "ThreadVolumeStatus started.");

	while (true)
	{
		Glib::Mutex::Lock lock(gmtxTimer);
		
		while (!gbUpdateStatus && !gbExit)
			gcondTimer.wait(gmtxTimer);
		
		if (gbExit)
			break;
		
		updateStatus();
		
		gbUpdateStatus = false;
	}
}

//==============================================================================

bool ThreadVolumeStatus::updateStatus()
{   
	if (!gmtxVolumeGuiLst.trylock())
		return false;

	for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
			ite != glstVolumeGui.end();
			++ite)
	{
		VolumeMountGui* poVolumeGui = (VolumeMountGui*)*ite;
		
		if (poVolumeGui->lockThread())
		{
			poVolumeGui->calcDiskUsage();
			poVolumeGui->lockThread(false);
		}
	}
	
	gmtxVolumeGuiLst.unlock();
	
	goDispatcher.emit();
	
	return true;
}

//==============================================================================
